global wayid;
global nodeid;
wayid = 2**30;
nodeid =2**30;

lon_range = 5:15;
lat_range = 47:55;

DEM = zeros(1200*length(lon_range)+1,1200*length(lat_range)+1);

for E=lon_range
	for N=lat_range
		% Read data
		FID = fopen(sprintf("N%02dE%03d.hgt",N,E));
		if FID < 0
			continue;
		end
		DEM((1200*(E-lon_range(1))+1):(1200*(E-lon_range(1))+1201), fliplr((1200*(N-lat_range(1))+1):(1200*(N-lat_range(1))+1201))) ...
			= reshape(fread(FID,Inf,'int16',0,'ieee-be'),1201,1201);
		fclose(FID);
	end
end
lon = lon_range(1):(1/1200):(lon_range(end)+1);
lat = lat_range(1):(1/1200):(lat_range(end)+1);

% Interpolate voids
% This algorithm isn't brilliant, but does fill in something
while true
	[ voids_lat voids_lon ] = spfind(sparse(DEM == -32768));
	if length(voids_lat) == 0;
		break;
	end

	for index = 1:length(voids_lat)
		if voids_lat(index) < 2 || voids_lat(index) > size(DEM,1)-2 || voids_lon(index) < 2 || voids_lon(index) > size(DEM,2)-2
			DEM(voids_lat(index), voids_lon(index)) = 0;
			continue;
		end
		local_DEM = DEM(voids_lat(index) + [ -1 0 1 ], ...
		                voids_lon(index) + [ -1 0 1 ]);
		local_DEM(local_DEM == -32768) = NaN;			                
		new_value = nanmean(local_DEM(1:9));
		if new_value == NaN
			continue;
		end
		DEM(voids_lat(index), voids_lon(index)) = new_value;
	end
end
% Save memory
DEM = uint16(DEM);
DEMfilt = zeros(size(DEM));

FID = fopen("contourdata.osm","w");
fprintf(FID,"<?xml version='1.0' encoding='UTF-8'?>\n");
fprintf(FID,"<osm version='0.5'>\n");

% The most zoomed-out setting should have a resolution of about 5 km
% The resolution of one data point is roughly 50 m, so the gauss kernel should have a 1/e-width of 100 pixels
% The width setting with the gausswin function is 2.5 times the 1/e-width, so we use 256 pixels there
kernel = gausswin(256);
kernel = kernel / sum(kernel);
for index = 1:size(DEM,1)
	DEMfilt(index,:) = fftfilt(kernel,DEM(index,:));
end
for index = 1:size(DEMfilt,2);
	DEMfilt(:,index) = fftfilt(kernel,DEMfilt(:,index));
end

% After filtering we downsample by a factor of 16 in each direction. Not more, because we want the contour lines to be smooth.
DEMdownsample = DEMfilt(1:16:size(DEMfilt,1),1:16:size(DEMfilt,2));
latdownsample = lat(1:16:length(lat));
londownsample = lon(1:16:length(lon));

C = contourc(latdownsample, londownsample, DEMdownsample, 200:200:4000);
write_contours_to_file(FID, '5', C)

clear C

% Next step has four times the resolution
kernel = gausswin(64);
kernel = kernel / sum(kernel);
for index = 1:size(DEM,1)
	DEMfilt(index,:) = fftfilt(kernel,DEM(index,:));
end
for index = 1:size(DEMfilt,2);
	DEMfilt(:,index) = fftfilt(kernel,DEMfilt(:,index));
end

% After filtering we downsample by a factor of 4 in each direction.
DEMdownsample = DEMfilt(1:4:size(DEMfilt,1),1:4:size(DEMfilt,2));
latdownsample = lat(1:4:length(lat));
londownsample = lon(1:4:length(lon));

C = contourc(latdownsample, londownsample, DEMdownsample, 100:100:4000);
write_contours_to_file(FID, '4', C)
clear C

% Next step has four times the resolution
kernel = gausswin(16);
kernel = kernel / sum(kernel);
for index = 1:size(DEM,1)
	DEMfilt(index,:) = fftfilt(kernel,DEM(index,:));
end
for index = 1:size(DEMfilt,2);
	DEMfilt(:,index) = fftfilt(kernel,DEMfilt(:,index));
end

% No downsampling here

C = contourc(lat, lon, DEMfilt, [ 100:100:500 600:100:4000 ]);
write_contours_to_file(FID, '3', C)
clear C

% Next step has four times the resolution
kernel = gausswin(4);
kernel = kernel / sum(kernel);
for index = 1:size(DEM,1)
	DEMfilt(index,:) = fftfilt(kernel,DEM(index,:));
end
for index = 1:size(DEMfilt,2);
	DEMfilt(:,index) = fftfilt(kernel,DEMfilt(:,index));
end

% No downsampling here

C = contourc(lat, lon, DEMfilt, [ 20:20:500 550:50:4000 ]);
write_contours_to_file(FID, '2', C)
clear C

fprintf(FID, "</osm>");
fclose(FID);
